@@ -1,3 +1,3 @@
 from . import actions
 from .base import exceptions
-from .queries import query
+from .queries import query, run
