@@ -270,6 +270,9 @@ class {{query.cls}}:
     def flat(self, selector: Callable[..., Iterable[R]] = None) -> "{{sequence.name}}[R]":
         return queries.Flat(self, selector)
 
+    def flat_recursive(self, selector: Callable[[{{query.row}}], Iterable[{{query.row}}]]) -> "{{sequence.name}}[{{query.row}}]":
+        return queries.FlatRecursive(self, selector)
+
     def unpack_pos(self, selector: Callable[..., R]) -> "{{sequence.name}}[R]":
         return queries.UnpackPos(self, selector=selector)
 
@@ -346,6 +349,9 @@ class {{query.cls}}:
     def take(self, count_or_range: Union[int, range]) -> "{{query.str}}":
         return queries.Take(self, count_or_range)
 
+    def take_while(self, predicate) -> "{{query.str}}":
+        return queries.TakeWhile(self, predicate)
+
     def skip(self, count_or_range: Union[int, range]) -> "{{query.str}}":
         return queries.Skip(self, count_or_range)
 
@@ -380,10 +386,10 @@ class {{query.cls}}:
     {% else %}
 class {{query.cls}}:
 
-    def get_many(self, *keys) -> "{{query.str}}":
+    def filter_keys(self, *keys) -> "{{query.str}}":
         raise NotImplementedError()
 
-    def must_get_many(self, *keys) -> "{{query.str}}":
+    def must_keys(self, *keys) -> "{{query.str}}":
         raise NotImplementedError()
 
     @overload
@@ -560,9 +566,13 @@ from .base import builder
 
 
 class QueryDict(queries.QueryDict):
-    def get_many(self, *keys):
+    def filter_keys(self, *keys):
         return queries.FilterKeys(self, *keys)
 
+    # @no_type_check
+    def must_keys(self, *keys):
+        return queries.MustKeys(self, *keys, typ="map")
+
     def get(self, key):
         try:
             return self.source[key]
@@ -586,15 +596,18 @@ class QueryDict(queries.QueryDict):
         else:
             return result
 
-    # @no_type_check
-    def must_get_many(self, *keys):
-        return queries.MustKeys(self, *keys, typ="map")
+
 
 
 class QuerySeq(queries.QuerySeq):
-    def get_many(self, *keys):
+    def filter_keys(self, *keys):
         return queries.FilterKeys(self, *keys)
 
+    # @no_type_check
+    def must_keys(self, *keys):
+        return queries.MustKeys(self, *keys, typ="seq")
+
+
     def get(self, key):
         try:
             return self.source[key]
@@ -618,17 +631,19 @@ class QuerySeq(queries.QuerySeq):
         else:
             return result
 
-    # @no_type_check
-    def must_get_many(self, *keys):
-        return queries.MustKeys(self, *keys, typ="seq")
+
 
 class QuerySet(queries.QuerySet):
     def order_by_reverse(self) -> "Query[T]":
         raise NotImplementedError("Set has no order.")
 
-    def get_many(self, *keys):
+    def filter_keys(self, *keys):
         return queries.FilterKeys(self, *keys)
 
+    # @no_type_check
+    def must_keys(self, *keys):
+        return queries.MustKeys(self, *keys, typ="set")
+
     def get(self, key):
         if key in self.source:
             return key
@@ -652,9 +667,8 @@ class QuerySet(queries.QuerySet):
         else:
             return result
 
-    # @no_type_check
-    def must_get_many(self, *keys):
-        return queries.MustKeys(self, *keys, typ="set")
+
+
 
 class QueryBuilder(builder.Builder):
     QUERY_BOTH = queries.Query
@@ -664,6 +678,7 @@ class QueryBuilder(builder.Builder):
     QUERY_DICT = QueryDict
     QUERY_SET = QuerySet
 
+
 @overload
 def query(source: Mapping[K, V]) -> DictEx[K, V]:
     ...
@@ -679,3 +694,4 @@ def query(source) -> "Query[Any]":
 def query(source):
     return QueryBuilder.query(source)
 
+run = QueryBuilder.run
\ No newline at end of file
