@@ -392,12 +392,6 @@ def select_single_node(
     pass
 
 
-@mark
-def map_recursive(self, selector):
-    "nodeを再帰的に取得する"
-    pass
-
-
 @mark
 def unpack_pos(self, selector):
     """シーケンスの各要素をアンパックし、新しいフォームに射影します。
@@ -552,7 +546,7 @@ def flat(self, selector=None):
     Args:
 
     * self: 変換対象のシーケンス
-    * selector: 各要素にから平坦化する要素を選択する関数
+    * selector: 各要素から平坦化する要素を選択する関数
 
     Returns: 参照したシーケンスをそのまま返す
 
@@ -571,6 +565,102 @@ select_many = flat
 flat_map = flat
 
 
+@mark
+def flat_recursive(self, selector):
+    """シーケンスの各要素から再帰的に複数ノードを選択し、選択されたノードを１つのシーケンスに平坦化します。
+    各ルート要素から浅い順に列挙されます。
+
+    Args:
+
+    * self: 変換対象のシーケンス
+    * selector: 各要素から平坦化する要素を再帰的に選択する関数（戻り値はリスト等に含めて返す必要があります）
+
+    Returns: 参照したシーケンスをそのまま返す
+
+    Usage:
+    ```
+    >>> pnq.query(
+    >>>     {"name": "a", "nodes": [{"name": "b", nodes: [{"name": c, "nodes": []}, {"name": "d", "nodes": []}}}]}]}
+    >>> ).flat_recursive(lambdax x: x["nodes"]).select("name").to(list)
+    >>> ["a", "b", "c", "d"]
+    ```
+    """
+
+
+def pivot_unstack(self, default=None):
+    """行方向に並んでいるデータを列方向に入れ替える
+
+    Args:
+
+    * self: 変換対象のシーケンス
+    * default: フィールドが存在しない場合のデフォルト値
+
+    Returns: 参照したシーケンスをそのまま返す
+
+    Usage:
+    ```
+    data = [
+        {"name": "test1", "age": 20},
+        {"name": "test2", "age": 25},
+        {"name": "test3", "age": 30, "sex": "male"},
+    ]
+    {'name': ['test1', 'test2', 'test3'], 'age': [20, 25, 30], 'sex': [None, None, 'male']}
+    ```
+    """
+    from collections import defaultdict
+
+    dataframe = {}
+    data = []
+
+    # 全てのカラムを取得
+    for i, dic in enumerate(self):
+        data.append(dic)
+        for k in dic.keys():
+            dataframe[k] = None
+
+    # カラム分の領域を初期化
+    for k in dataframe:
+        dataframe[k] = []
+
+    # データをデータフレームに収める
+    for dic in data:
+        for k in dataframe.keys():
+            v = dic.get(k, default)
+            dataframe[k].append(v)
+
+    yield from dataframe.items()
+
+
+def pivot_stack(self):
+    """列方向に並んでいるデータを行方向に入れ替える
+
+    Args:
+
+    * self: 変換対象のシーケンス
+
+    Returns: 参照したシーケンスをそのまま返す
+
+    Usage:
+    ```
+    {'name': ['test1', 'test2', 'test3'], 'age': [20, 25, 30], 'sex': [None, None, 'male']}
+    data = [
+        {"name": "test1", "age": 20, "sex": None},
+        {"name": "test2", "age": 25, "sex": None},
+        {"name": "test3", "age": 30, "sex": "male"},
+    ]
+    ```
+    """
+    data = dict(self)
+    columns = list(data.keys())
+
+    for i in range(len(columns)):
+        row = {}
+        for c in columns:
+            row[c] = data[c][i]
+
+        yield row
+
+
 @mark
 def cast(self, type):
     """シーケンスの型注釈を変更します。この関数はエディタの型解釈を助けるためだけに存在し、何も処理を行いません。
@@ -639,7 +729,7 @@ def group_by(self, selector=lambda x: x):
     >>> ]
     >>> pnq.query(data).group_by(lambda x: x["color"], x["name"]).to(list)
     [("yellow", ["banana"]), ("red", ["apple", "strawberry"])]
-    >>> pnq.query(data).select("color", "count").group_by().to_dict()
+    >>> pnq.query(data).select("color", "count").group_by().to(dict)
     {"yellow": [3], "red": [2, 5]}
     ```
     """
@@ -662,59 +752,6 @@ def group_join(self, right, on, select):
     pass
 
 
-def pivot_unstack(self, default=None):
-    """行方向に並んでいるデータを列方向に入れ替える
-    data = [
-        {"name": "test1", "age": 20},
-        {"name": "test2", "age": 25},
-        {"name": "test3", "age": 30, "sex": "male"},
-    ]
-    {'name': ['test1', 'test2', 'test3'], 'age': [20, 25, 30], 'sex': [None, None, 'male']}
-    """
-    from collections import defaultdict
-
-    dataframe = {}
-    data = []
-
-    # 全てのカラムを取得
-    for i, dic in enumerate(self):
-        data.append(dic)
-        for k in dic.keys():
-            dataframe[k] = None
-
-    # カラム分の領域を初期化
-    for k in dataframe:
-        dataframe[k] = []
-
-    # データをデータフレームに収める
-    for dic in data:
-        for k in dataframe.keys():
-            v = dic.get(k, default)
-            dataframe[k].append(v)
-
-    yield from dataframe.items()
-
-
-def pivot_stack(self):
-    """列方向に並んでいるデータを行方向に入れ替える
-    {'name': ['test1', 'test2', 'test3'], 'age': [20, 25, 30], 'sex': [None, None, 'male']}
-    data = [
-        {"name": "test1", "age": 20, "sex": None},
-        {"name": "test2", "age": 25, "sex": None},
-        {"name": "test3", "age": 30, "sex": "male"},
-    ]
-    """
-    data = dict(self)
-    columns = list(data.keys())
-
-    for i in range(len(columns)):
-        row = {}
-        for c in columns:
-            row[c] = data[c][i]
-
-        yield row
-
-
 @mark
 def request(self, func, retry: int = None):
     """シーケンスから流れてくる値を同期関数に送出するように要求します。
@@ -738,9 +775,9 @@ def request(self, func, retry: int = None):
     >>>
     >>> for res in pnq.query([{"id": 1, "val": True}, {"id": 2, "val": False}]).request(do_something):
     >>>   if res.err:
-    >>>     print(f"ERROR: {res.to_dict()}")
+    >>>     print(f"ERROR: {res.to(dict)}")
     >>>   else:
-    >>>     print(f"SUCCESS: {res.to_dict()}")
+    >>>     print(f"SUCCESS: {res.to(dict)}")
     ```
     """
     from .requests import Response, StopWatch
@@ -788,9 +825,9 @@ async def request_async(self, func, timeout: float = None, retry: int = None):
     >>> params = pnq.query([{"id": 1, "val": True}, {"id": 2, "val": False}])
     >>> async for res in params.request_async(do_something):
     >>>   if res.err:
-    >>>     print(f"ERROR: {res.to_dict()}")
+    >>>     print(f"ERROR: {res.to(dict)}")
     >>>   else:
-    >>>     print(f"SUCCESS: {res.to_dict()}")
+    >>>     print(f"SUCCESS: {res.to(dict)}")
     ```
     """
     from .requests import Response, StopWatch
@@ -973,6 +1010,68 @@ def must_type(self, types):
         yield elm
 
 
+@mark
+def filter_keys(self, *keys):
+    """シーケンスの要素から指定したキーの要素のみフィルタ処理します。
+    このメソッドは、`list` `dict` `set`などをクエリ化した直後のみ利用できます。
+
+    * `list`、`tuple`の場合インデックスでフィルタされ、値を返します。
+    * `dict`の場合キーでフィルタされ、キーと要素を返します。
+    * `set`の場合は、キーでフィルタされ、キーを返します。
+
+    Args:
+
+    * self: フィルタ対象のシーケンス
+    * keys: フィルタするキー
+
+    Returns: 指定したキーの要素のみ返すクエリ
+
+    Usage:
+    ```
+    >>> pnq.query([1, 2]).filter_keys(1).to(list)
+    [2]
+    >>> pnq.query({"a": 1, "b": 2}).filter_keys("b").to(list)
+    [("b", 2)]
+    >>> pnq.query({"a", "b"}).filter_keys("b").to(list)
+    ["b"]
+    ```
+    """
+
+
+@mark
+def must_keys(self, *keys):
+    """`filter_keys`を実行し、全てのキーを取得できなかった場合例外を発生させます。
+    検証が完了するまで、ストリームは保留されます。
+    """
+
+    not_exists = set()
+    key_values = []
+    raise_if_not_unique_keys(keys)
+    undefined = object()
+
+    for k in keys:
+        val = get(self, k, undefined)
+        if val is undefined:
+            not_exists.add(k)
+        else:
+            key_values.append((k, val))
+
+    if not_exists:
+        raise NotFoundError(str(not_exists))
+
+    if typ == "map":
+        for k, v in key_values:
+            yield k, v
+    elif typ == "seq":
+        for k, v in key_values:
+            yield v
+    elif typ == "set":
+        for k, v in key_values:
+            yield k
+    else:
+        raise TypeError(f"unknown type: {typ}")
+
+
 @mark
 def filter_unique(self, selector=None):
     """シーケンスの要素から重複する要素を除去する。
@@ -1042,81 +1141,6 @@ def raise_if_not_unique_keys(keys):
         raise TypeError(f"can't accept duplicate keys: {keys}")
 
 
-@mark
-def get_many(self, *keys):
-    # undefined = object()
-    # raise_if_not_unique_keys(keys)
-    # for id in keys:
-    #     obj = get_or(self, id, undefined)
-    #     if obj is not undefined:
-    #         yield id, obj
-    ...
-
-
-def get_many_for_mapping(self, *keys):
-    """"""
-    undefined = object()
-    raise_if_not_unique_keys(keys)
-    for id in keys:
-        obj = get_or(self, id, undefined)
-        if obj is not undefined:
-            yield id, obj
-
-
-def get_many_for_sequence(self, *keys):
-    """"""
-    undefined = object()
-    raise_if_not_unique_keys(keys)
-    for id in keys:
-        obj = get_or(self, id, undefined)
-        if obj is not undefined:
-            yield obj
-
-
-def get_many_for_set(self, *keys):
-    """"""
-    undefined = object()
-    raise_if_not_unique_keys(keys)
-    for id in keys:
-        obj = get_or(self, id, undefined)
-        if obj is not undefined:
-            yield id
-
-
-@mark
-def must_get_many(self, *keys, typ: Literal["set", "seq", "map"]):
-    """`get_many`を実行し、全てのキーを取得できなかった場合例外を発生させます。
-    検証が完了するまで、ストリームは保留されます。
-    """
-
-    not_exists = set()
-    key_values = []
-    raise_if_not_unique_keys(keys)
-    undefined = object()
-
-    for k in keys:
-        val = get(self, k, undefined)
-        if val is undefined:
-            not_exists.add(k)
-        else:
-            key_values.append((k, val))
-
-    if not_exists:
-        raise NotFoundError(str(not_exists))
-
-    if typ == "map":
-        for k, v in key_values:
-            yield k, v
-    elif typ == "seq":
-        for k, v in key_values:
-            yield v
-    elif typ == "set":
-        for k, v in key_values:
-            yield k
-    else:
-        raise TypeError(f"unknown type: {typ}")
-
-
 ###########################################
 # partitioning
 ###########################################
@@ -1147,6 +1171,7 @@ def take(self, count_or_range: int):
 @mark
 def take_while(self, predicate):
     """シーケンスの先頭から、条件の検証に失敗するまでの要素を返します。
+    **検証に失敗した要素は破棄されるため、値を消費するイテレータがソースの場合は注意が必要です。**
 
     Args:
 
