@@ -286,6 +286,38 @@ class Flat(Query):
             return (_ async for elm in self.source async for _ in selector(elm))
 
 
+@mark
+class FlatRecursive(Query):
+    def __init__(self, source, selector):
+        super().__init__(source)
+
+        def scan(parent, selector):
+            nodes = selector(parent)
+            if nodes is None:
+                return
+            for node in nodes:
+                yield node
+                yield from scan(node, selector)
+
+        self.scan = scan
+        self.selector = selector
+
+    def _impl_iter(self):
+        scan = self.scan
+        selector = self.selector
+        for node in self.source:
+            yield node
+            yield from scan(node, selector)
+
+    async def _impl_aiter(self):
+        scan = self.scan
+        selector = self.selector
+        async for node in self.source:
+            yield node
+            for c in scan(node, selector):
+                yield c
+
+
 @mark
 class Enumerate(Query):
     def __init__(self, source, start: int = 0, step: int = 1):
@@ -332,13 +364,6 @@ class GroupBy(Query):
             yield k, v
 
 
-@mark
-class GroupCount(Query):
-    """collections.Counter"""
-
-    pass
-
-
 @mark
 class PivotUnstack(Query):
     def __init__(self, source, default=None) -> None:
@@ -812,6 +837,29 @@ class Take(Query):
             return
 
 
+@mark
+class TakeWhile(Query):
+    def __init__(self, source, predicate):
+        super().__init__(source)
+        self.predicate = predicate
+
+    def _impl_iter(self):
+        predicate = self.predicate
+        for v in self.source:
+            if predicate(v):
+                yield v
+            else:
+                break
+
+    async def _impl_aiter(self):
+        predicate = self.predicate
+        async for v in self.source:
+            if predicate(v):
+                yield v
+            else:
+                break
+
+
 @mark
 class Skip(Query):
     def __init__(self, source, count_or_range: Union[int, range]):
