@@ -260,6 +260,9 @@ class Query(Generic[T]):
     def flat(self, selector: Callable[..., Iterable[R]] = None) -> "Query[R]":
         return queries.Flat(self, selector)
 
+    def flat_recursive(self, selector: Callable[[T], Iterable[T]]) -> "Query[T]":
+        return queries.FlatRecursive(self, selector)
+
     def unpack_pos(self, selector: Callable[..., R]) -> "Query[R]":
         return queries.UnpackPos(self, selector=selector)
 
@@ -337,6 +340,9 @@ class Query(Generic[T]):
     def take(self, count_or_range: Union[int, range]) -> "Query[T]":
         return queries.Take(self, count_or_range)
 
+    def take_while(self, predicate) -> "Query[T]":
+        return queries.TakeWhile(self, predicate)
+
     def skip(self, count_or_range: Union[int, range]) -> "Query[T]":
         return queries.Skip(self, count_or_range)
 
@@ -612,6 +618,11 @@ class PairQuery(Generic[K, V]):
     def flat(self, selector: Callable[..., Iterable[R]] = None) -> "Query[R]":
         return queries.Flat(self, selector)
 
+    def flat_recursive(
+        self, selector: Callable[[Tuple[K, V]], Iterable[Tuple[K, V]]]
+    ) -> "Query[Tuple[K,V]]":
+        return queries.FlatRecursive(self, selector)
+
     def unpack_pos(self, selector: Callable[..., R]) -> "Query[R]":
         return queries.UnpackPos(self, selector=selector)
 
@@ -691,6 +702,9 @@ class PairQuery(Generic[K, V]):
     def take(self, count_or_range: Union[int, range]) -> "PairQuery[K,V]":
         return queries.Take(self, count_or_range)
 
+    def take_while(self, predicate) -> "PairQuery[K,V]":
+        return queries.TakeWhile(self, predicate)
+
     def skip(self, count_or_range: Union[int, range]) -> "PairQuery[K,V]":
         return queries.Skip(self, count_or_range)
 
@@ -729,10 +743,10 @@ class PairQuery(Generic[K, V]):
 
 
 class IndexQuery(Generic[K, V]):
-    def get_many(self, *keys) -> "IndexQuery[K,V]":
+    def filter_keys(self, *keys) -> "IndexQuery[K,V]":
         raise NotImplementedError()
 
-    def must_get_many(self, *keys) -> "IndexQuery[K,V]":
+    def must_keys(self, *keys) -> "IndexQuery[K,V]":
         raise NotImplementedError()
 
     @overload
@@ -904,9 +918,13 @@ from .base import builder
 
 
 class QueryDict(queries.QueryDict):
-    def get_many(self, *keys):
+    def filter_keys(self, *keys):
         return queries.FilterKeys(self, *keys)
 
+    # @no_type_check
+    def must_keys(self, *keys):
+        return queries.MustKeys(self, *keys, typ="map")
+
     def get(self, key):
         try:
             return self.source[key]
@@ -930,15 +948,15 @@ class QueryDict(queries.QueryDict):
         else:
             return result
 
-    # @no_type_check
-    def must_get_many(self, *keys):
-        return queries.MustKeys(self, *keys, typ="map")
-
 
 class QuerySeq(queries.QuerySeq):
-    def get_many(self, *keys):
+    def filter_keys(self, *keys):
         return queries.FilterKeys(self, *keys)
 
+    # @no_type_check
+    def must_keys(self, *keys):
+        return queries.MustKeys(self, *keys, typ="seq")
+
     def get(self, key):
         try:
             return self.source[key]
@@ -962,18 +980,18 @@ class QuerySeq(queries.QuerySeq):
         else:
             return result
 
-    # @no_type_check
-    def must_get_many(self, *keys):
-        return queries.MustKeys(self, *keys, typ="seq")
-
 
 class QuerySet(queries.QuerySet):
     def order_by_reverse(self) -> "Query[T]":
         raise NotImplementedError("Set has no order.")
 
-    def get_many(self, *keys):
+    def filter_keys(self, *keys):
         return queries.FilterKeys(self, *keys)
 
+    # @no_type_check
+    def must_keys(self, *keys):
+        return queries.MustKeys(self, *keys, typ="set")
+
     def get(self, key):
         if key in self.source:
             return key
@@ -997,10 +1015,6 @@ class QuerySet(queries.QuerySet):
         else:
             return result
 
-    # @no_type_check
-    def must_get_many(self, *keys):
-        return queries.MustKeys(self, *keys, typ="set")
-
 
 class QueryBuilder(builder.Builder):
     QUERY_BOTH = queries.Query
@@ -1028,3 +1042,6 @@ def query(source) -> "Query[Any]":
 
 def query(source):
     return QueryBuilder.query(source)
+
+
+run = QueryBuilder.run
